# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: middle-steps.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='middle-steps.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12middle-steps.proto\"&\n\nMiddleStep\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\t\"0\n\x0bMiddleSteps\x12!\n\x0cmiddle_steps\x18\x01 \x03(\x0b\x32\x0b.MiddleStepb\x06proto3'
)




_MIDDLESTEP = _descriptor.Descriptor(
  name='MiddleStep',
  full_name='MiddleStep',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='MiddleStep.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='MiddleStep.data', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22,
  serialized_end=60,
)


_MIDDLESTEPS = _descriptor.Descriptor(
  name='MiddleSteps',
  full_name='MiddleSteps',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='middle_steps', full_name='MiddleSteps.middle_steps', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=62,
  serialized_end=110,
)

_MIDDLESTEPS.fields_by_name['middle_steps'].message_type = _MIDDLESTEP
DESCRIPTOR.message_types_by_name['MiddleStep'] = _MIDDLESTEP
DESCRIPTOR.message_types_by_name['MiddleSteps'] = _MIDDLESTEPS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MiddleStep = _reflection.GeneratedProtocolMessageType('MiddleStep', (_message.Message,), {
  'DESCRIPTOR' : _MIDDLESTEP,
  '__module__' : 'middle_steps_pb2'
  # @@protoc_insertion_point(class_scope:MiddleStep)
  })
_sym_db.RegisterMessage(MiddleStep)

MiddleSteps = _reflection.GeneratedProtocolMessageType('MiddleSteps', (_message.Message,), {
  'DESCRIPTOR' : _MIDDLESTEPS,
  '__module__' : 'middle_steps_pb2'
  # @@protoc_insertion_point(class_scope:MiddleSteps)
  })
_sym_db.RegisterMessage(MiddleSteps)


# @@protoc_insertion_point(module_scope)
